@@ -1,6 +1,6 @@
 /*
  *
- * $Id: context.h,v 4.25 2000/11/04 00:17:14 hubert Exp $
+ * $Id: context.h,v 4.30 2004/07/01 17:27:20 hubert Exp $
  *
  * Program:	Mailbox Context Management
  *
@@ -18,7 +18,7 @@
  * permission of the University of Washington.
  *
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2000 by the University of Washington.
+ * 1989-2004 by the University of Washington.
  *
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -42,8 +42,16 @@ char	   *context_digest PROTO((char *, char *, char *, char *, char *,
 				  size_t));
 char	   *context_apply PROTO((char *, CONTEXT_S *, char *, size_t));
 long	    context_create PROTO((CONTEXT_S *, MAILSTREAM *, char *));
-MAILSTREAM *context_open PROTO((CONTEXT_S *, MAILSTREAM *, char *, long));
+MAILSTREAM *context_open PROTO((CONTEXT_S *, MAILSTREAM *, char *, long,
+				long *));
 long	    context_status PROTO((CONTEXT_S *, MAILSTREAM *, char *, long));
+long	    context_status_full PROTO((CONTEXT_S *, MAILSTREAM *, char *,
+				       long, unsigned long *, unsigned long *));
+long	    context_status_streamp PROTO((CONTEXT_S *, MAILSTREAM **,
+					  char *, long));
+long	    context_status_streamp_full PROTO((CONTEXT_S *, MAILSTREAM **,
+					  char *, long,
+					  unsigned long *, unsigned long *));
 long	    context_rename PROTO((CONTEXT_S *, MAILSTREAM *, char *, char *));
 long	    context_delete PROTO((CONTEXT_S *, MAILSTREAM *, char *));
 long	    context_append PROTO((CONTEXT_S *, MAILSTREAM *, char *, \
@@ -51,8 +59,9 @@ long	    context_append PROTO((CONTEXT_S *, MAILSTREAM *, char *, \
 long	    context_append_full PROTO((CONTEXT_S *, MAILSTREAM *, char *, \
 				       char *, char *, STRING *));
 long	    context_append_multiple PROTO((CONTEXT_S *, MAILSTREAM *, char *, \
-					   append_t, void *));
+					   append_t, void *, MAILSTREAM *));
 long	    context_copy PROTO((CONTEXT_S *, MAILSTREAM *, char *, char *));
 MAILSTREAM *context_same_stream PROTO((CONTEXT_S *, char *, MAILSTREAM *));
+MAILSTREAM *context_already_open_stream PROTO((CONTEXT_S *, char *, int));
 
 #endif /* _CONTEXT_INCLUDED */
