@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: help.c,v 4.124 2001/08/07 22:49:47 hubert Exp $";
+static char rcsid[] = "$Id: help.c,v 4.138 2004/11/23 18:29:47 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: help.c,v 4.124 2001/08/07 22:49:47 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-2001 by the University of Washington.
+   1989-2004 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -122,6 +122,9 @@ static char att_cur_msg[] = "\
 int	 helper_internal PROTO((HelpType, char **, char *, char *, int));
 HelpType help_name2section PROTO((char *, int));
 int	 help_processor PROTO((int, MSGNO_S *, SCROLL_S *));
+int	 journal_processor PROTO((int, MSGNO_S *, SCROLL_S *));
+void     prune_review_messages PROTO((size_t *, size_t));
+size_t   trim_review PROTO((unsigned long, int, size_t));
 void	 help_keymenu_tweek PROTO((SCROLL_S *, int));
 int	 help_bogus_input PROTO((int));
 char   **get_supported_options PROTO((void));
@@ -218,13 +221,13 @@ helper_internal(text, otext, frag, title, flags)
 	    if(!struncmp(shown_text[0], "<html>", 6))
 	      gf_link_filter(gf_html2plain,
 			     gf_html2plain_opt("x-pine-help:",
-					ps_global->ttyo->screen_cols,
-					&handles, GFHP_LOCAL_HANDLES));
+					       ps_global->ttyo->screen_cols, NULL,
+					       &handles, GFHP_LOCAL_HANDLES));
 	    else
 	      gf_link_filter(gf_wrap, gf_wrap_filter_opt(
 						  ps_global->ttyo->screen_cols,
 						  ps_global->ttyo->screen_cols,
-						  0, GFW_HANDLES));
+						  NULL, 0, GFW_HANDLES));
 
 	    error = gf_pipe(helper_getc, pc);
 
@@ -350,7 +353,8 @@ helper_internal(text, otext, frag, title, flags)
 		    clrbitn(HLP_NEXT_HANDLE, sargs.keys.bitmap);
 		}
 
-		if(text != main_menu_tx) /* only main can "print all" */
+		if(text != main_menu_tx
+		   && text != h_mainhelp_pinehelp)
 		  clrbitn(HLP_ALL_KEY, sargs.keys.bitmap);
 
 		if(frag){
@@ -527,7 +531,8 @@ print_help(text)
 	char *p;
 
 	gf_link_filter(gf_html2plain,
-		       gf_html2plain_opt(NULL,80,NULL,GFHP_STRIPPED));
+		       gf_html2plain_opt(NULL,80,NULL,
+					 NULL,GFHP_STRIPPED));
 	for(i = 1; i <= 5 && text[i]; i++)
 	  if(!struncmp(text[i], "<title>", 7)
 	     && (p = srchstr(text[i] + 7, "</title>"))
@@ -538,7 +543,7 @@ print_help(text)
 	  }
     }
     else
-      gf_link_filter(gf_wrap, gf_wrap_filter_opt(80, 80, 0, 0));
+      gf_link_filter(gf_wrap, gf_wrap_filter_opt(80, 80, NULL, 0, 0));
 
     gf_link_filter(gf_line_test,
 		   gf_line_test_opt(print_help_page_break, NULL));
@@ -551,7 +556,7 @@ print_help(text)
     print_text(NEWLINE);
 
     if(error = gf_pipe(helper_getc, print_char))
-      q_status_message1(SM_ORDER | SM_DING, 3, 3, "Printing Error: %s", error);
+      q_status_message1(SM_ORDER | SM_DING, 3, 3, "Printing Error: %.200s", error);
 
     print_char(ctrl('L'));		/* new page. */
 }
@@ -689,7 +694,7 @@ url_local_helper(url)
     }
 
     q_status_message1(SM_ORDER | SM_DING, 0, 3,
-		      "Unrecognized Internal help: \"%s\"", url);
+		      "Unrecognized Internal help: \"%.200s\"", url);
     return(0);
 }
 
@@ -801,7 +806,7 @@ url_local_config(url)
     }
 
     q_status_message1(SM_ORDER | SM_DING, 0, 3,
-		      "Unrecognized Internal help: \"%s\"", url);
+		      "Unrecognized Internal help: \"%.200s\"", url);
     return(0);
 }
 
@@ -870,7 +875,7 @@ help_name2section(url, url_len)
 	}
 	else
 	  q_status_message1(SM_ORDER | SM_DING, 3, 5,
-			    "No Help!  Index \"%s\" not found.", buf);
+			    "No Help!  Index \"%.200s\" not found.", buf);
 
     }
 #else
@@ -907,7 +912,7 @@ get_help_text(index)
     build_path(buf, ps_global->pine_dir, HELPINDEX, sizeof(buf));
     if(!(helpfile = fopen(buf, "rb"))){
 	q_status_message1(SM_ORDER,3,5,
-	    "No Help!  Index \"%s\" not found.", buf);
+	    "No Help!  Index \"%.200s\" not found.", buf);
 	return(NULL);
     }
 
@@ -924,7 +929,7 @@ get_help_text(index)
     /* make sure help file is open */
     build_path(buf, ps_global->pine_dir, HELPFILE, sizeof(buf));
     if((helpfile = fopen(buf, "rb")) == NULL){
-	q_status_message2(SM_ORDER,3,5,"No Help!  \"%s\" : %s", buf,
+	q_status_message2(SM_ORDER,3,5,"No Help!  \"%.200s\" : %.200s", buf,
 			  error_description(errno));
 	return(NULL);
     }
@@ -971,149 +976,424 @@ get_help_text(index)
 #endif	/* HELPFILE */
 
 
+#define RMMSGLEN 120
+#define RMTIMLEN 15
+#define RMJLEN   500
+#define RMLLEN   2000
+#define RMHLEN   2000
+typedef struct _rev_msg {
+    unsigned long    seq;
+    short            level;	/* -1 for journal, debuglevel for dprint */
+    unsigned         continuation:1;
+    char             message[RMMSGLEN+1];
+    char             timestamp[RMTIMLEN+1];
+} REV_MSG_S;
 
+typedef enum {No, Jo, Lo, Hi} RMCat;
 
-#if defined(DOS) && !defined(_WINDOWS)
-#define NSTATUS 25  /* how many status messages to save for review */
-#else
-#define NSTATUS 100
-#endif
+static REV_MSG_S rmjoarray[RMJLEN];	/* For regular journal */
+static REV_MSG_S rmloarray[RMLLEN];	/* debug 0-4 */
+static REV_MSG_S rmhiarray[RMHLEN];	/* debug 5-9 */
+static int       rmjofirst = -1, rmjolast = -1;
+static int       rmlofirst = -1, rmlolast = -1;
+static int       rmhifirst = -1, rmhilast = -1;
+static unsigned long rmseq = 0L;
 
-static char *stat_msgs[NSTATUS];
-static int   latest;
+static int        not_right_now;
 
 /*----------------------------------------------------------------------
-     Review last N status messages
-
-    Args: title  -- The title of the screen
+     Review latest status messages
   -----*/
 void
-review_messages(title)
-    char  *title;
+review_messages()
 {
-#define INDENT 2
-    int             how_many_lines, how_many_bytes = 0, width, resetwidth;
-    char           *tmp_text, *cur;
-    char           *p, *q, *last_space, *cp;
-    char            save, buf[MAX_SCREEN_COLS + 1];
-    register char **e;
-    register int    i, k, m, line_len;
     SCROLL_S	    sargs;
+    STORE_S        *in_store = NULL, *out_store = NULL;
+    gf_io_t         gc, pc;
+    int             jo, lo, hi, donejo, donelo, donehi;
+    RMCat           rmcat;
+    int             cmd, timestamps=0, show_level=-1;
+    char            debugkeylabel[20];
+    char            timestampkeylabel[] = "NoTimestamps";
+    static struct key rev_msg_keys[] =
+       {HELP_MENU,
+	NULL_MENU,
+	{"E","Exit Viewer",{MC_EXIT,1,{'e'}},KS_EXITMODE},
+	NULL_MENU,
+	{"T",NULL,{MC_TOGGLE,1,{'t'}},KS_NONE},
+	{"D",NULL,{MC_JUMP,1,{'d'}},KS_NONE},
+	PREVPAGE_MENU,
+	NEXTPAGE_MENU,
+	PRYNTTXT_MENU,
+	WHEREIS_MENU,
+	FWDEMAIL_MENU,
+	{"S", "Save", {MC_SAVETEXT,1,{'s'}}, KS_SAVE}};
+    INST_KEY_MENU(rev_msg_keymenu, rev_msg_keys);
+#define TIMESTAMP_KEY 4
+#define DEBUG_KEY     5
 
-    e = stat_msgs;
-    width = resetwidth = max(20, ps_global->ttyo->screen_cols);
+    if((rmjofirst < 0 && rmlofirst < 0 && rmhifirst < 0)
+       || rmjofirst >= RMJLEN || rmjolast >= RMJLEN
+       || rmlofirst >= RMLLEN || rmlolast >= RMLLEN
+       || rmhifirst >= RMHLEN || rmhilast >= RMHLEN
+       || (rmjofirst >= 0 && rmjolast < 0)
+       || (rmlofirst >= 0 && rmlolast < 0)
+       || (rmhifirst >= 0 && rmhilast < 0))
+      return;
+    
+    do{
 
-    /* conservative estimate of space we'll use */
-    for(i = 0; i < NSTATUS; i++)
-      if(e[i] && *e[i]){
-	  line_len = strlen(e[i]);
-	  how_many_lines = line_len/(width - INDENT) + 1;
-	  how_many_bytes += (line_len + how_many_lines +
-					      (how_many_lines-1) * INDENT);
-      }
+	if(!(in_store = so_get(CharStar, NULL, EDIT_ACCESS)) ||
+	   !(out_store = so_get(CharStar, NULL, EDIT_ACCESS))){
+	    if(in_store)
+	      so_give(&in_store);
 
-    cur = tmp_text = (char *)fs_get((how_many_bytes + 1) * sizeof(char *));
-    *cur = '\0';
+	    q_status_message(SM_ORDER | SM_DING, 3, 4,
+			     "Failed allocating memory");
+	    return;
+	}
+	
+	add_review_message("Turning off new messages while reviewing", 0);
+	not_right_now = 1;
 
-    /* allocate strings */
-    for(k = 0, i = latest; k < NSTATUS; k++){
-	i = (i + 1) % NSTATUS;
-	if(e[i] && *e[i]){
-	    p = q = e[i];
-	    last_space = NULL;
-	    line_len   = 0;
-	    width      = resetwidth;
-	    while(*p){
-		if(*p == TAB){
-		    last_space = p++;
-		    while(line_len < width
-			  && ((++line_len)&0x07) != 0)
-		      ;
-		}
-		else if(*p == SPACE){
-		    last_space = p++;
-		    line_len++;
-		}
-		else{
-		    p++;
-		    line_len++;
-		}
+	donejo = donehi = donelo = 0;
+	jo = rmjofirst;
+	if(jo < 0)
+	  donejo = 1;
 
-		if(line_len > width){
-		    if(width != resetwidth){
-			for(m = 0; m < INDENT; m++)
-			  buf[m] = SPACE;
-			
-			cp = buf + INDENT;
-		    }
-		    else
-		      cp = buf;
+	lo = rmlofirst;
+	if(lo < 0)
+	  donelo = 1;
 
-		    if(last_space){
-			save = *last_space;
-			*last_space = '\0';
-			strncpy(cp, q,
-				(width != resetwidth) ? sizeof(buf)-INDENT
-						      : sizeof(buf));
-			buf[sizeof(buf)-1] = '\0';
-			sstrcpy(&cur, buf);
-			sstrcpy(&cur, "\n");
-			*last_space = save;
-			q = last_space + 1;
-		    }
-		    else{
-			save = q[width];
-			q[width] = '\0';
-			strncpy(cp, q,
-				(width != resetwidth) ? sizeof(buf)-INDENT
-						      : sizeof(buf));
-			buf[sizeof(buf)-1] = '\0';
-			sstrcpy(&cur, buf);
-			sstrcpy(&cur, "\n");
-			q[width] = save;
-			q = q + width;
+	hi = rmhifirst;
+	if(hi < 0)
+	  donehi = 1;
+
+	while(!(donejo && donelo && donehi)){
+	    REV_MSG_S *pjo, *plo, *phi, *p;
+
+	    if(!donejo)
+	      pjo = &rmjoarray[jo];
+	    else
+	      pjo = NULL;
+
+	    if(!donelo)
+	      plo = &rmloarray[lo];
+	    else
+	      plo = NULL;
+
+	    if(!donehi)
+	      phi = &rmhiarray[hi];
+	    else
+	      phi = NULL;
+
+	    if(pjo && (!plo || pjo->seq <= plo->seq)
+	       && (!phi || pjo->seq <= phi->seq))
+	      rmcat = Jo;
+	    else if(plo && (!phi || plo->seq <= phi->seq))
+	      rmcat = Lo;
+	    else if(phi)
+	      rmcat = Hi;
+	    else
+	      rmcat = No;
+
+	    switch(rmcat){
+	      case Jo:
+		p = pjo;
+		if(jo == rmjofirst && (((rmjolast + 1) % RMJLEN) == rmjofirst))
+		  so_puts(in_store,
+	"**** Journal entries prior to this point have been trimmed. ****\n");
+		break;
+
+	      case Lo:
+		p = plo;
+		if(show_level >= 0 &&
+		   lo == rmlofirst && (((rmlolast + 1) % RMLLEN) == rmlofirst))
+		  so_puts(in_store,
+	"**** Debug 0-4 entries prior to this point have been trimmed. ****\n");
+		break;
+
+	      case Hi:
+		p = phi;
+		if(show_level >= 5 &&
+		   hi == rmhifirst && (((rmhilast + 1) % RMHLEN) == rmhifirst))
+		  so_puts(in_store,
+	"**** Debug 5-9 entries prior to this point have been trimmed. ****\n");
+		break;
+
+	      default:
+		p = NULL;
+		break;
+	    }
+
+	    if(p){
+		if(p->level <= show_level){
+		    if(timestamps && p->timestamp && p->timestamp[0]){
+			so_puts(in_store, p->timestamp);
+			so_puts(in_store, ": ");
 		    }
 
-		    p = q;
-		    line_len = 0;
-		    last_space = NULL;
-		    width = resetwidth - 2;
+		    if(p->message && p->message[0]){
+			if(p->continuation)
+			  so_puts(in_store, ">");
+
+			so_puts(in_store, p->message);
+			so_puts(in_store, "\n");
+		    }
 		}
 	    }
 
-	    if(*q){
-		if(width != resetwidth){
-		    for(m = 0; m < INDENT; m++)
-		      buf[m] = SPACE;
-		    
-		    cp = buf + INDENT;
-		}
+	    switch(rmcat){
+	      case Jo:
+		if(jo == rmjolast)
+		  donejo++;
 		else
-		  cp = buf;
+		  jo = (jo + 1) % RMJLEN;
 
-		strncpy(cp, q,
-			(width != resetwidth) ? sizeof(buf)-INDENT
-					      : sizeof(buf));
-		buf[sizeof(buf)-1] = '\0';
-		sstrcpy(&cur, buf);
-		sstrcpy(&cur, "\n");
+		break;
+
+	      case Lo:
+		if(lo == rmlolast)
+		  donelo++;
+		else
+		  lo = (lo + 1) % RMLLEN;
+
+		break;
+
+	      case Hi:
+		if(hi == rmhilast)
+		  donehi++;
+		else
+		  hi = (hi + 1) % RMHLEN;
+
+		break;
+
+	      default:
+		donejo++;
+		donelo++;
+		donehi++;
+		break;
+	    }
+	}
+
+
+	so_seek(in_store, 0L, 0);
+	gf_filter_init();
+	gf_link_filter(gf_wrap,
+		       gf_wrap_filter_opt(ps_global->ttyo->screen_cols - 4,
+					  ps_global->ttyo->screen_cols,
+					  NULL, show_level < 0 ? 2 : 0, 0));
+	gf_set_so_readc(&gc, in_store);
+	gf_set_so_writec(&pc, out_store);
+	gf_pipe(gc, pc);
+	gf_clear_so_writec(out_store);
+	gf_clear_so_readc(in_store);
+
+	memset(&sargs, 0, sizeof(SCROLL_S));
+	sargs.text.text     = so_text(out_store);
+	sargs.text.src      = CharStar;
+	sargs.text.desc     = "journal";
+	sargs.keys.menu     = &rev_msg_keymenu;
+	sargs.proc.tool     = journal_processor;
+	sargs.start.on      = LastPage;
+	sargs.resize_exit   = 1;
+	sargs.proc.data.p   = (void *)&show_level;
+	setbitmap(sargs.keys.bitmap);
+
+#ifdef DEBUG
+#ifdef DEBUGJOURNAL
+	sargs.jump_is_debug = 1;
+	sargs.help.title    = "HELP FOR DEBUG JOURNAL";
+	sargs.help.text     = h_debugjournal;
+	sargs.bar.title     = "REVIEW DEBUGGING";
+#else	/* !DEBUGJOURNAL */
+	clrbitn(DEBUG_KEY, sargs.keys.bitmap);
+	sargs.help.title    = "HELP FOR JOURNAL";
+	sargs.help.text     = h_journal;
+	sargs.bar.title     = "REVIEW RECENT MESSAGES";
+#endif	/* !DEBUGJOURNAL */
+#else	/* !DEBUG */
+	clrbitn(DEBUG_KEY, sargs.keys.bitmap);
+	clrbitn(TIMESTAMP_KEY, sargs.keys.bitmap);
+	sargs.help.title    = "HELP FOR JOURNAL";
+	sargs.help.text     = h_journal;
+	sargs.bar.title     = "REVIEW RECENT MESSAGES";
+#endif	/* !DEBUG */
+
+	if(timestamps)
+	  rev_msg_keys[TIMESTAMP_KEY].label = timestampkeylabel;
+	else
+	  rev_msg_keys[TIMESTAMP_KEY].label = timestampkeylabel + 2;
+
+	if(show_level >= 0)
+	  sprintf(debugkeylabel, "Debug (%d)", show_level);
+	else
+	  strncpy(debugkeylabel, "DebugView", sizeof(debugkeylabel));
+
+	rev_msg_keys[DEBUG_KEY].label = debugkeylabel;
+	KS_OSDATASET(&rev_msg_keys[DEBUG_KEY], KS_NONE);
+
+	if((cmd = scrolltool(&sargs)) == MC_TOGGLE)
+	  timestamps = !timestamps;
+
+	so_give(&in_store);
+	so_give(&out_store);
+	
+    }while(cmd != MC_EXIT);
+
+    not_right_now = 0;
+    add_review_message("Done reviewing", 0);
+}
+
+
+int
+journal_processor(cmd, msgmap, sparms)
+    int	      cmd;
+    MSGNO_S  *msgmap;
+    SCROLL_S *sparms;
+{
+    switch(cmd){
+      case MC_TOGGLE:		/* turn timestamps on or off */
+        break;
+
+      default:
+        panic("Unexpected command in journal_processor");
+	break;
+    }
+    
+    return(1);
+}
+
+
+#ifdef DEBUG
+void
+debugjournal_to_file(dfile)
+    FILE *dfile;
+{
+    int donejo, donelo, donehi, jo, lo, hi;
+    RMCat rmcat;
+
+    if(dfile && (rmjofirst >= 0 || rmlofirst >= 0 || rmhifirst >= 0)
+       && rmjofirst < RMJLEN && rmjolast < RMJLEN
+       && rmlofirst < RMLLEN && rmlolast < RMLLEN
+       && rmhifirst < RMHLEN && rmhilast < RMHLEN
+       && (rmjofirst < 0 || rmjolast >= 0)
+       && (rmlofirst < 0 || rmlolast >= 0)
+       && (rmhifirst < 0 || rmhilast >= 0)){
+
+	donejo = donehi = donelo = 0;
+	jo = rmjofirst;
+	if(jo < 0)
+	  donejo = 1;
+
+	lo = rmlofirst;
+	if(lo < 0)
+	  donelo = 1;
+
+	hi = rmhifirst;
+	if(hi < 0)
+	  donehi = 1;
+
+	while(!(donejo && donelo && donehi)){
+	    REV_MSG_S *pjo, *plo, *phi, *p;
+
+	    if(!donejo)
+	      pjo = &rmjoarray[jo];
+	    else
+	      pjo = NULL;
+
+	    if(!donelo)
+	      plo = &rmloarray[lo];
+	    else
+	      plo = NULL;
+
+	    if(!donehi)
+	      phi = &rmhiarray[hi];
+	    else
+	      phi = NULL;
+
+	    if(pjo && (!plo || pjo->seq <= plo->seq)
+	       && (!phi || pjo->seq <= phi->seq))
+	      rmcat = Jo;
+	    else if(plo && (!phi || plo->seq <= phi->seq))
+	      rmcat = Lo;
+	    else if(phi)
+	      rmcat = Hi;
+	    else
+	      rmcat = No;
+
+	    if(rmcat == Jo){
+		p = pjo;
+		if(jo == rmjofirst &&
+		   (((rmjolast + 1) % RMJLEN) == rmjofirst) &&
+       fputs("*** Level -1 entries prior to this are deleted", dfile) == EOF)
+		  break;
+	    }
+	    else if(rmcat == Lo){
+		p = plo;
+		if(lo == rmlofirst &&
+		   (((rmlolast + 1) % RMLLEN) == rmlofirst) &&
+       fputs("*** Level 0-4 entries prior to this are deleted", dfile) == EOF)
+		  break;
+	    }
+	    else if(rmcat == Hi){
+		p = phi;
+		if(hi == rmhifirst &&
+		   (((rmhilast + 1) % RMHLEN) == rmhifirst) &&
+       fputs("*** Level 5-9 entries prior to this are deleted", dfile) == EOF)
+		  break;
+	    }
+	    else if(rmcat == No){
+		p = NULL;
+	    }
+
+	    if(p){
+		if(p->timestamp && p->timestamp[0]
+		   && (fputs(p->timestamp, dfile) == EOF
+		       || fputs(": ", dfile) == EOF))
+		  break;
+
+		if(p->message && p->message[0]
+		   && (fputs(p->message, dfile) == EOF
+		       || fputs("\n", dfile) == EOF))
+		  break;
+	    }
+
+	    switch(rmcat){
+	      case Jo:
+		if(jo == rmjolast)
+		  donejo++;
+		else
+		  jo = (jo + 1) % RMJLEN;
+
+		break;
+
+	      case Lo:
+		if(lo == rmlolast)
+		  donelo++;
+		else
+		  lo = (lo + 1) % RMLLEN;
+
+		break;
+
+	      case Hi:
+		if(hi == rmhilast)
+		  donehi++;
+		else
+		  hi = (hi + 1) % RMHLEN;
+
+		break;
+
+	      default:
+		donejo++;
+		donelo++;
+		donehi++;
+		break;
 	    }
 	}
     }
-
-    *cur = '\0';
-
-    memset(&sargs, 0, sizeof(SCROLL_S));
-    sargs.text.text = tmp_text;
-    sargs.text.src  = CharStar;
-    sargs.text.desc = "journal";
-    sargs.bar.title = title;
-    sargs.start.on  = LastPage;
-
-    scrolltool(&sargs);
-
-    fs_give((void **)&tmp_text);
 }
+#endif /* DEBUG */
 
 
 /*----------------------------------------------------------------------
@@ -1122,36 +1402,101 @@ review_messages(title)
     Args: message  -- The message to add
   -----*/
 void
-add_review_message(message)
+add_review_message(message, level)
     char *message;
+    int   level;
 {
-    if(!(message && *message))
+    int   next_is_continuation = 0, cur_is_continuation = 0;
+    char *p, *q;
+
+    if(not_right_now || !(message && *message))
       return;
 
-    latest = (latest + 1) % NSTATUS;
-    if(stat_msgs[latest] && strlen(stat_msgs[latest]) >= strlen(message))
-      strcpy(stat_msgs[latest], message);  /* already enough space */
-    else{
-	if(stat_msgs[latest])
-	  fs_give((void **)&stat_msgs[latest]);
+    /*
+     * Debug output can have newlines in it, so split up each newline piece
+     * by hand and make them separate messages.
+     */
+    not_right_now = 1;
+    for(p = message; *p; p = (*q && !next_is_continuation) ? q+1 : q){
+	for(q = p; *q && *q != '\n' && (q-p) < RMMSGLEN; q++)
+	  ;
 
-	stat_msgs[latest] = cpystr(message);
+	if(p == q)
+	  continue;
+
+	cur_is_continuation = next_is_continuation;
+
+	if((q-p) == RMMSGLEN && *q && *q != '\n')
+	  next_is_continuation = 1;
+	else
+	  next_is_continuation = 0;
+
+	if(level < 0){
+	    if(rmjofirst < 0){
+		rmjofirst = 0;
+		rmjolast  = 0;
+	    }
+	    else{
+		rmjolast = (rmjolast + 1) % RMJLEN;
+		if(rmjolast == rmjofirst)
+		  rmjofirst = (rmjofirst + 1) % RMJLEN;
+	    }
+
+	    rmjoarray[rmjolast].level = (short) level;
+	    rmjoarray[rmjolast].seq   = rmseq++;
+	    rmjoarray[rmjolast].continuation = cur_is_continuation ? 1 : 0;
+	    memset(rmjoarray[rmjolast].message, 0, (RMMSGLEN+1)*sizeof(char));
+	    strncpy(rmjoarray[rmjolast].message, p, min(q-p,RMMSGLEN));
+#ifdef DEBUG
+	    memset(rmjoarray[rmjolast].timestamp, 0, (RMTIMLEN+1)*sizeof(char));
+	    strncpy(rmjoarray[rmjolast].timestamp, debug_time(0,1), RMTIMLEN);
+#endif
+	}
+	else if(level <= 4){
+	    if(rmlofirst < 0){
+		rmlofirst = 0;
+		rmlolast  = 0;
+	    }
+	    else{
+		rmlolast = (rmlolast + 1) % RMLLEN;
+		if(rmlolast == rmlofirst)
+		  rmlofirst = (rmlofirst + 1) % RMLLEN;
+	    }
+
+	    rmloarray[rmlolast].level = (short) level;
+	    rmloarray[rmlolast].seq   = rmseq++;
+	    rmloarray[rmlolast].continuation = cur_is_continuation ? 1 : 0;
+	    memset(rmloarray[rmlolast].message, 0, (RMMSGLEN+1)*sizeof(char));
+	    strncpy(rmloarray[rmlolast].message, p, min(q-p,RMMSGLEN));
+#ifdef DEBUG
+	    memset(rmloarray[rmlolast].timestamp, 0, (RMTIMLEN+1)*sizeof(char));
+	    strncpy(rmloarray[rmlolast].timestamp, debug_time(0,1), RMTIMLEN);
+#endif
+	}
+	else{
+	    if(rmhifirst < 0){
+		rmhifirst = 0;
+		rmhilast  = 0;
+	    }
+	    else{
+		rmhilast = (rmhilast + 1) % RMHLEN;
+		if(rmhilast == rmhifirst)
+		  rmhifirst = (rmhifirst + 1) % RMHLEN;
+	    }
+
+	    rmhiarray[rmhilast].level = (short) level;
+	    rmhiarray[rmhilast].seq   = rmseq++;
+	    rmhiarray[rmhilast].continuation = cur_is_continuation ? 1 : 0;
+	    memset(rmhiarray[rmhilast].message, 0, (RMMSGLEN+1)*sizeof(char));
+	    strncpy(rmhiarray[rmhilast].message, p, min(q-p,RMMSGLEN));
+#ifdef DEBUG
+	    memset(rmhiarray[rmhilast].timestamp, 0, (RMTIMLEN+1)*sizeof(char));
+	    strncpy(rmhiarray[rmhilast].timestamp, debug_time(0,1), RMTIMLEN);
+#endif
+	}
     }
-}
 
-
-
-/*----------------------------------------------------------------------
-    Free resources associated with the status message review list
-
-    Args: 
-  -----*/
-void
-end_status_review()
-{
-    for(latest = NSTATUS - 1; latest >= 0; latest--)
-      if(stat_msgs[latest])
-	fs_give((void **)&stat_msgs[latest]);
+    not_right_now = 0;
 }
 
 
@@ -1381,10 +1726,10 @@ gripe_newbody(ps, body, msgno, flags)
 					 NULL, NULL,
 					 PIPE_READ | PIPE_STDERR | PIPE_USER,
 					 0)){
-		    gf_set_readc(&gc, (void *)syspipe->in.f, 0, FileStar);
+		    gf_set_readc(&gc, (void *)syspipe, 0, PipeStar);
 		    gf_filter_init();
 		    error = gf_pipe(gc, pc);
-		    (void) close_system_pipe(&syspipe);
+		    (void) close_system_pipe(&syspipe, NULL, 0);
 		}
 		else
 		  error = "executing config collector";
@@ -1394,7 +1739,7 @@ gripe_newbody(ps, body, msgno, flags)
 	    
 	    if(error){
 		q_status_message1(SM_ORDER | SM_DING, 3, 4, 
-				  "Problem %s", error);
+				  "Problem %.200s", error);
 		return(-1);
 	    }
 	    else			/* fixup attachment's size */
@@ -1429,7 +1774,8 @@ gripe_newbody(ps, body, msgno, flags)
 		}
 
 		/* write the header */
-		if((p = mail_fetchheader(ps->mail_stream, msgno)) && *p)
+		if((p = mail_fetch_header(ps->mail_stream, msgno, NIL, NIL,
+					  NIL, FT_PEEK)) && *p)
 		  so_puts(store, p);
 		else
 		  return(-1);
@@ -1442,7 +1788,8 @@ gripe_newbody(ps, body, msgno, flags)
 		/* HACK!  See mailview.c:format_message for details... */
 		ps->mail_stream->text = NULL;
 		/* write the body */
-		if(!mail_fetchtext(ps->mail_stream, msgno))
+		if(!pine_mail_fetch_text(ps->mail_stream, msgno,
+					 NULL, NULL, NIL))
 		  return(-1);
 
 		pb->size.bytes = ftell(append_file);
@@ -1455,7 +1802,9 @@ gripe_newbody(ps, body, msgno, flags)
 		pb->size.bytes = strlen(p);
 		so_puts(store, "\015\012");
 
-		if((p = mail_fetchtext(ps->mail_stream, msgno)) &&  *p)
+		if((p = pine_mail_fetch_text(ps->mail_stream, 
+					     msgno, NULL, NULL, NIL))
+		   &&  *p)
 		  so_puts(store, p);
 		else
 		  return(-1);
