@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: newmail.c,v 4.151 2004/07/14 18:16:52 hubert Exp $";
+static char rcsid[] = "$Id: newmail.c,v 4.156 2005/01/14 00:43:13 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -56,6 +56,7 @@ int  check_point PROTO((MAILSTREAM *, CheckPointTime, int));
 void new_mail_mess PROTO((MAILSTREAM *, long, long, int));
 void new_mail_win_mess PROTO((MAILSTREAM *, long));
 void fixup_flags PROTO((MAILSTREAM *, MSGNO_S *));
+void newmailfifo PROTO((int, char *, char *, char *));
 
 
 /*----------------------------------------------------------------------
@@ -520,6 +521,9 @@ new_mail(force_arg, time_for_check_point, flags)
 #ifdef _WINDOWS
 		if(mswin_newmailwinon())
 		  new_mail_win_mess(m, t_nm_count);
+#elif !defined(DOS) && !defined(OS2) && !defined(LEAVEOUTFIFO)
+		if(ps_global->VAR_FIFOPATH)
+		  new_mail_win_mess(m, t_nm_count);
 #endif
 		if(n)
 		  new_mail_mess(m, sp_mail_since_cmd(m), n, 0);
@@ -568,6 +572,9 @@ new_mail(force_arg, time_for_check_point, flags)
 #ifdef _WINDOWS
 		    if(mswin_newmailwinon())
 		      new_mail_win_mess(m, t_nm_count);
+#elif !defined(DOS) && !defined(OS2) && !defined(LEAVEOUTFIFO)
+		    if(ps_global->VAR_FIFOPATH)
+		      new_mail_win_mess(m, t_nm_count);
 #endif
 		    if(n)
 		      new_mail_mess(m, sp_mail_since_cmd(m), n, 0);
@@ -589,6 +596,18 @@ new_mail(force_arg, time_for_check_point, flags)
           maybe_kill_old_stream(m);
     }
 
+    /*
+     * This is here to prevent banging on the down arrow key (or holding
+     * it down and repeating) at the end of the index from causing
+     * a whole bunch of new mail checks. Last_nextitem_forcechk does get
+     * set at the place it is tested in mailcmd.c, but this is here to
+     * reset it to a little bit later in case it takes a second or more
+     * to check for the mail. If we didn't do this, we'd just check every
+     * keystroke as long as the checks took more than a second.
+     */
+    if(force_arg)
+      ps_global->last_nextitem_forcechk = time(0);
+
     dprint(6, (debugfile, "******** new mail returning %ld  ********\n", 
 	       rv ? rv : (exp_count ? 0 : -1)));
     return(rv ? rv : (exp_count ? 0 : -1));
@@ -776,7 +795,7 @@ new_mail_mess(stream, number, max_num, for_new_mail_win)
 				     : "",
  		      (number <= 1L) ? (subject[0] ? "" : "ith no subject")
 				     : "");
-#ifdef _WINDOWS
+#if (!defined(DOS) && !defined(OS2) && !defined(LEAVEOUTFIFO)) || defined(_WINDOWS)
     else {
 	int is_us = 0;
 	ADDRESS *tadr;
@@ -787,14 +806,33 @@ new_mail_mess(stream, number, max_num, for_new_mail_win)
 		is_us = 1;
 		break;
 	    }
+#ifdef _WINDOWS
 	mswin_newmailwin(is_us, from, subject, folder);
+#else
+	newmailfifo(is_us, from, subject, folder);
+#endif
     }
 #endif
 
-    sprintf(tmp_20k_buf, "%s%s%s%.80s", intro,
-	    from ? ((number > 1L) ? " Most recent f" : " F") : "",
-	    from ? "rom " : "",
-	    from ? from : "");
+    if(F_ON(F_ENABLE_XTERM_NEWMAIL, ps_global)
+       && F_ON(F_ENABLE_NEWMAIL_SHORT_TEXT, ps_global)){
+	long inbox_nm;
+	if(!sp_flagged(stream, SP_INBOX) 
+	   && (inbox_nm = sp_mail_since_cmd(sp_inbox_stream()))){
+	    sprintf(tmp_20k_buf, "[%d, %d] %s",
+		    inbox_nm > 1L ? inbox_nm : 1,
+		    number > 1L ? number: 1,
+		    ps_global->pine_name);
+	}
+	else
+	  sprintf(tmp_20k_buf, "[%d] %s", number > 1L ? number: 1,
+		  ps_global->pine_name);
+    }
+    else
+      sprintf(tmp_20k_buf, "%s%s%s%.80s", intro,
+	      from ? ((number > 1L) ? " Most recent f" : " F") : "",
+	      from ? "rom " : "",
+	      from ? from : "");
     icon_text(tmp_20k_buf, IT_NEWMAIL);
 
     if(from)
@@ -806,6 +844,7 @@ new_mail_mess(stream, number, max_num, for_new_mail_win)
 
 
 
+
 /*----------------------------------------------------------------------
   Straighten out any local flag problems here.  We can't take care of
   them in the mm_exists or mm_expunged callbacks since the flags
@@ -974,7 +1013,7 @@ check_point(stream, timing, flags)
 	else{
 	    dprint(9, (debugfile,
 	      "Skipping checkpoint since last was only %ld secs ago (< %d)\n",
-	      time(0) - sp_last_chkpnt_done(stream), AT_LEAST));
+	      (long) (time(0) - sp_last_chkpnt_done(stream)), AT_LEAST));
 	}
     }
 
@@ -1124,3 +1163,139 @@ streams_died()
 
     return(rv);
 }
+
+
+#if !defined(DOS) && !defined(OS2) && !defined(LEAVEOUTFIFO)
+static char *fifoname = NULL;
+static int   fifoopenerrmsg = 0;
+static int   fifofd = -1;
+static int   fifoheader = 0;
+
+void
+init_newmailfifo(fname)
+    char *fname;
+{
+    if(fifoname)
+      close_newmailfifo();
+
+    if(!(fname && *fname))
+      return;
+
+    if(!fifoname){
+	if(mkfifo(fname, 0600) == -1){
+	    q_status_message2(SM_ORDER,3,3,
+			      "Can't create NewMail FIFO \"%s\": %s",
+			      fname, error_description(errno));
+	    return;
+	}
+	else
+	  q_status_message1(SM_ORDER,0,3, "NewMail FIFO: \"%s\"", fname);
+
+	fifoname = cpystr(fname);
+    }
+}
+
+
+void
+close_newmailfifo()
+{
+    if(fifoname){
+	if(fifofd >= 0)
+	  (void) close(fifofd);
+
+	if(*fifoname)
+	  (void) unlink(fifoname);
+
+	fs_give((void **) &fifoname);
+    }
+
+    fifoheader = 0;
+    fifoname = NULL;
+    fifofd = -1;
+    fifoopenerrmsg = 0;
+}
+
+
+void
+newmailfifo(is_us, from, subject, folder)
+    int   is_us;
+    char *from,
+	 *subject,
+	 *folder;
+{
+    char buf[MAX_SCREEN_COLS+1], buf2[MAX_SCREEN_COLS+1];
+    char buf3[MAX_SCREEN_COLS+1], buf4[MAX_SCREEN_COLS+1];
+
+    if(!(fifoname && *fifoname)){
+	if(fifoname)
+	  close_newmailfifo();
+
+	return;
+    }
+
+    if(fifofd < 0){
+	fifofd = open(fifoname, O_WRONLY | O_NONBLOCK);
+	if(fifofd < 0){
+	    if(!fifoopenerrmsg){
+		if(errno == ENXIO)
+		  q_status_message2(SM_ORDER,0,3, "Nothing reading \"%s\": %s",
+				    fifoname, error_description(errno));
+		else
+		  q_status_message2(SM_ORDER,0,3, "Can't open \"%s\": %s",
+				    fifoname, error_description(errno));
+
+		fifoopenerrmsg++;
+	    }
+
+	    return;
+	}
+    }
+
+    if(fifofd >= 0){
+	int width;
+	int fromlen, subjlen, foldlen;
+
+	width = min(max(20, ps_global->nmw_width), MAX_SCREEN_COLS);
+
+	foldlen = .18 * width;
+	foldlen = max(5, foldlen);
+	fromlen = .28 * width;
+	subjlen = width - 2 - foldlen - fromlen;
+
+	if(!fifoheader){
+	    time_t now;
+	    char *tmtxt;
+
+	    now = time((time_t *) 0);
+	    tmtxt = ctime(&now);
+	    if(!tmtxt)
+	      tmtxt = "";
+
+	    sprintf(buf, "New Mail window started at %.*s\n",
+		    min(100, strlen(tmtxt)-1), tmtxt);
+	    (void) write(fifofd, buf, strlen(buf));
+
+	    sprintf(buf, "  %-*s%-*s%-*s\n",
+		    fromlen, "From:",
+		    subjlen, "Subject:",
+		    foldlen, "Folder:");
+	    (void) write(fifofd, buf, strlen(buf));
+
+	    sprintf(buf, "%-*.*s\n", width, width, repeat_char(width, '-'));
+	    (void) write(fifofd, buf, strlen(buf));
+
+	    fifoheader++;
+	}
+
+	sprintf(buf, "%s %-*.*s %-*.*s %-*.*s\n", is_us ? "+" : " ",
+		fromlen - 1, fromlen - 1,
+		short_str(from ? from : "", buf2, fromlen-1, EndDots),
+		subjlen - 1, subjlen - 1,
+		short_str(subject ? subject : "(no subject)",
+			  buf3, subjlen-1, EndDots),
+		foldlen, foldlen,
+		short_str(folder ? folder : "INBOX", buf4, foldlen, FrontDots));
+	(void) write(fifofd, buf, strlen(buf));
+    }
+}
+#endif
