@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-mnt.h,v 1.8 2001/02/07 00:52:58 hubert Exp $
+  $Id: os-mnt.h,v 1.10 2003/05/23 16:05:21 hubert Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -109,11 +109,13 @@
 /*----------------------------------------------------------------------
     Timeouts (seconds)
  ----*/
+#ifndef DF_MAILCHECK
 #define DF_MAILCHECK      "150" /* How often to check for new mail, by
 				   default.  There's some expense in doing
 				   this so it shouldn't be done too
 				   frequently.  (Can be set in config
 				   file now.)  */
+#endif
 
 /*----------------------------------------------------------------------
     Check pointing (seconds)
@@ -160,8 +162,12 @@
 
 
 /*----- System-wide config file ----------------------------------------*/
+#ifndef SYSTEM_PINERC
 #define SYSTEM_PINERC             "/usr/local/lib/pine.conf"
+#endif
+#ifndef SYSTEM_PINERC_FIXED
 #define SYSTEM_PINERC_FIXED       "/usr/local/lib/pine.conf.fixed"
+#endif
 
 
 
@@ -211,9 +217,13 @@
 
 
 /*----- The usual sendmail configuration for sending mail on Unix ------*/
+#ifndef SENDMAIL
 #define SENDMAIL	"/usr/sbin/sendmail"
+#endif
+#ifndef SENDMAILFLAGS
 #define SENDMAILFLAGS	"-bs -odb -oem"	/* send via smtp with backgroud
 					   delivery and mail back errors */
+#endif
 
 
 /*----------------------------------------------------------------------
@@ -223,7 +233,9 @@
 
 
 /*--------- Program employed by users to change their password ---------*/
+#ifndef	PASSWD_PROG
 #define	PASSWD_PROG	"/bin/passwd"
+#endif
 
 
 /*-------------- A couple constants used to size arrays ----------------*/
@@ -276,7 +288,7 @@
     Various maximum field lengths, probably shouldn't be changed.
  ----*/
 #define MAX_FULLNAME     (100) 
-#define MAX_NICKNAME      (40)
+#define MAX_NICKNAME      (80)
 #define MAX_ADDRESS      (500)
 #define MAX_NEW_LIST     (500)  /* Max addrs to be added when creating list */
 #define MAX_SEARCH       (100)  /* Longest string to search for             */
